% Matlab code required for computing the numerical examples in 4 of 
% "Sparse FFT for Functions with Short Frequency Support" by Sina Bittens

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [A] = exact_eval(n,B,nu,coeff)
% EXACT_EVAL Returns A=(f(2*pi*a/n)), a=0:n-1, and its Fourier transform 
% Ahat for a function f with short support length B<=N, where nu is the 
% first frequency in the support and Coeff is the B-length vector of 
% Fourier coefficients.

% Initialize Ahat, the vector consisting of the Fourier coefficients of
% f: Ahat(j+1) is the sum over all Fourier coefficients whose
% corresponding frequencies are congruent to j modulo n.
Ahat=zeros(n,1);
for j=0:B-1
        % Since f is a function with support length B<=n, the sums consist 
        % of at most one summand and the at most B energetic frequencies 
        % are consecutive.
        Ahat(mod(nu+j,n)+1)=coeff(j+1);
end
% The values f(2*pi*a/n) are given by n times the inverse Fourier transform 
% of Ahat due to MATLAB's normalization constants.
A=n*ifft(Ahat);
end