% Matlab code required for computing the numerical examples in 4 of 
% "Sparse FFT for Functions with Short Frequency Support" by Sina Bittens

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [A,Ahat] = exact_eval_N(N,B,nu,Coeff)
% EXACT_EVAL_N Returns A=(f(2*pi*a/N)), a=0:N-1, and its Fourier transform 
% Ahat for a function f with short support length B<=N, and even bandwidth 
% N, where nu is the first frequency in the support and Coeff is the 
% B-length vector of Fourier coefficients. 

% Ahat is given by inserting the Fourier coefficients of f at the position
% corresponding to their respective frequencies in an N-length vector,
% where the first entry corresponds to the frequency -N/2+1.
if (mod(N,2)==0)
    Ahat=zeros(N,1);
    for a=1:B
        Ahat(nu+N/2-1+(a-1)+1)=Coeff(a);
    end
    % The values f(2*pi*a/N) are given by the inverse Fourier transform of
    % Ahat due to MATLAB's normalization constants.
    A=ifft(Ahat);
else
    disp('N has to be even.')
end
end