% Required for the Matlab code example for Algorithm 1 presented in
% "Sparse FFT for Functions with Short Frequency Support" by Sina Bittens

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [L,p] = findL(B,N)
%FINDL Find a natural number L such that the product of the smallest L-1 
% odd primes satisfies p_2*...*p_(L-1)<N/B<=p_2*...*p_(L-1)*p_L, where p_l 
% is the l-th prime number. Return L and the vector p=[1 p_1 ... p_{L-1}].

% Initialize L=2, the current prime p=3 and the product P=p of all 
% previously considered odd primes.
L=2; 
p=3; 
P=p;
% While the product of the first L-1 odd primes is smaller than N/B, the 
% next prime, the first prime greater than p, has to be included as well.
while (P<N/B)
    % Increase L.
    L=L+1;
    % Compute the first prime greater than p.
    p=nprimes(1,p+1);
    % Include it in the product.
    P=p*P; 
end
% Compute the L-1 smallest odd primes.
p=[1 nprimes(L-1,3)];
% By choice of L we have L=length(p) and p starts with [1 3 5 ...].
end