% Matlab code required for computing the numerical examples in 4 of 
% "Sparse FFT for Functions with Short Frequency Support" by Sina Bittens

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [m,p_sparse] = findm(B,N)
%FINDM Find a natural number m such that the product of the smallest m 
% primes satisfies N/B<=p_1*...*p_m, where p_l is the l-th prime number. 
% Return m and the vector p_sparse=[1 p_1 ... p_m].

% Initialize m=1, the current prime p=2 and the product P=p of all 
% previously considered primes.
m=1; 
p=2; 
P=p;
% While the product of the first m primes is smaller than N/B, the next 
% prime, the first prime greater than p, has to be included as well.
while (P<N/B)
    % Increase m.
    m=m+1;
    % Compute the first prime greater than p.
    p=nprimes(1,p+1);
    % Include it in the product.
    P=p*P; 
end
% Compute the m smallest primes.
p_sparse=[1 nprimes(m,2)];
% By choice of m we have m+1=length(p) and p starts with [1 2 3 5 ...].
end