% Matlab code required for computing the numerical examples in 4 of 
% "Sparse FFT for Functions with Short Frequency Support" by Sina Bittens

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [A] = noisy_eval(n,N,coeff)
%NOISY_EVAL Returns A=(f(2*pi*a/n)), a=0:n-1, for a non-sparse function f 
% with short support length B<=N and even bandwidth N, where nu is the 
% first frequency in the support and Coeff is the B-length vector of 
% Fourier coefficients.

if (mod(N,2)==0)
    % Initialize Ahat, the vector consisting of the Fourier coefficients of
    % f: Ahat(j+1) is the sum over all Fourier coefficients whose
    % corresponding frequencies are congruent to j modulo n.
    Ahat=zeros(n,1);
    r=mod(-N/2+1,n);
    for j=1:n-r
            % Since f is a block function with B<=n, the sums consist of at
            % most one summand and the non-empty entries are consecutive.
            Ahat(r+j)=sum(coeff(j:n:end));
    end
    for j=n-r+1:n
        Ahat(j+r-n)=sum(coeff(j:n:end));
    end
    % The values f(2*pi*a/n) are given by the inverse Fourier transform of
    % Ahat (MATLAB has different normalization constants).
    A=n*ifft(Ahat);
else
    disp('N has to be even.')
end
end

