% Required for the Matlab code example for Algorithm 1 presented in
% "Sparse FFT for Functions with Short Frequency Support" by Sina Bittens

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [p] = nprimes(m,s)
%NPRIMES Returns a vector p consisting of the m smallest primes greater 
% than or equal to floor(s).

% Initialise the output vector p, the counter for the number of found 
% primes and j, the next natural number that will be checked for primeness,
% starting at floor(s).
p=zeros(1,m); 
counter=0; 
j=floor(s);
% While less than m primes have been found, check whether the next odd 
% number j is prime. If this is the case, add j to the output vector and 
% update the counter.
% If we start at j>2 even, j is not a prime and it suffices to check j+1.
if (mod(j,2)==0) && (j>2)
    j=j+1;
end
% If j is 1 or 2, the next natural number to be checked is j+1.
while (counter<m) && (j<=2)
    if isprime(j)
        counter=counter+1;
        p(counter)=j;
    end
    j=j+1;
end
% Now j is always odd and greater than 2, so if j is prime, the next prime 
% is odd as well and the difference between it and j is at least 2, so the
% next number to be checked is j+2.
while counter<m
    if isprime(j)
        counter=counter+1;
        p(counter)=j;
    end
    j=j+2;
end
end

