% Required for the Matlab code example for Algorithm 1 presented in
% "Sparse FFT for Functions with Short Frequency Support" by Sina Bittens

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [nu] = reconstruction(r,p,N)
%RECONSTRUCTION Reconstruct nu in {-N/2+1,...,N/2} from its residues r(l) 
% modulo p(l) (see ''ChineseRem'' in GAP).
if length(r)~=length(p)
    disp('size(r) not equal to size(p)')
elseif mod(N,2)==1
    disp('N has to be even.')
else
    m=length(r)-1;
    % Initialise nu as the first residue r(1), the current product modulus 
    % n as the first modulus p(1) and the index l for the m+1 entries of r 
    % and p.
    nu=r(1); 
    n=p(1);
    l=1;
    % Find nu such that it successively satisfies nu=r(l) mod p(l) for all 
    % l<=m+1.
    while l<m+1
        l=l+1;
        [g,u,~]=exgcd(n,p(l));
        % Test whether it is possible that nu=r(l) mod p(l) while 
        % satisfying the congruencies nu=r(j) mod p(j) for j<l.
        if (g~=1) && (mod(r(l)-nu,g)~=0)
            disp('Error: The residues have to be equal modulo g')
        end
        % Update nu to satisfy the congruencies for all r(j) and p(j) where 
        % j<=l and update the product modulus n leaving out the common 
        % divisors of p(l) and n.
        nu=n*(mod((r(l)-nu)/g*u,p(l)))+nu;
        n=p(l)/g*n;
    end
    % Shift nu into {-n/2+1,...,n/2} for the final modulus n which is 
    % usually greater than the bandwidth N needed in SFFT_short_support.
    if (0<=mod(nu,n)) && (mod(nu,n)<=floor(n/2))
        nu=mod(nu,n);
    else
        nu=mod(nu,n)-n;
    end
    % Shift nu into {-N/2+1,...,N/2} in order to comply with the 
    % requirements of SFFT_short_support.
    while (nu>floor(N/2))
        nu=nu-N;
    end
    while (nu<-(floor(N/2)+1)+1)
        nu=nu+N;
    end
end  
end

